==> ec2init-0.11-py3-none-any.whl: ec2init/plans/init/start_services.py <==
import logging
import configparser
import pathlib

from typing import List, Tuple

from ec2init import Procedure
from ec2init.utils.system import SystemUtils
from ec2init.utils.dns import DnsUtils
from ec2init.plans.init.callback_hopsworks import CallbackHopsworks
from ec2init.utils.cloud import get_cloud_provider

logger = logging.getLogger(__name__)


class StartServices(Procedure):
    SYSTEMCTL_ENABLE = "systemctl enable {service}"
    SYSTEMCTL_START = "systemctl start {service}"
    
    def __init__(self, name: str, gateway: str, instance_id: str, node_id: str, instance_type: str,
                 uid: str, config: configparser.ConfigParser, report_endpoint: Tuple[str, str], api_key: str, cloud_provider: str, search_domain: str) -> None:
        self.name = name
        self.gateway = gateway
        self.instance_id = instance_id
        self.uid = uid
        self.config = config
        self.report_endpoint = report_endpoint
        self.api_key = api_key
        self.node_id = node_id
        self.instance_type = instance_type
        self.cloud_provider = cloud_provider
        self.search_domain = search_domain

    def pre(self) -> None:
        if self.report_callback is not None:
            try:
                self.report_callback.execute()
            except Exception as ex:
                logger.warning(
                    "%s - Could not execute pre-execute: %s", self.name, ex)

    def execute(self) -> List[Procedure]:
        logger.info("%s - Starting execution", self.name)

        # Wait until internal hostname is resolved successfully
        # Get my FQDN
        fqdn = get_cloud_provider(self.cloud_provider).get_fqdn(self.config, self.search_domain, self.node_id)
        DnsUtils.get_ip_from_name(fqdn)

        kagent_conf_dir = self.config.get(
            'general', 'kagent_conf_dir', fallback="/srv/hops/kagent/etc")
        services_file = pathlib.Path(kagent_conf_dir, "services")
        services = configparser.ConfigParser()
        services.read(str(services_file))
        for section in services.sections():
            service = services.get(
                section, 'service', fallback='service_does_not_exist')
            if service != 'service_does_not_exist':
                logger.debug("%s - Starting service: %s", self.name, service)
                self._start_service(service)
                logger.info("%s - Started service: %s", self.name, service)
                logger.debug("%s - Enabling service: %s", self.name, service)
                self._enable_service(service)
                logger.info("%s - Enabled service: %s", self.name, service)
                logger.debug("%s - Reloading Consul", self.name)
                self._reload_consul()
        logger.info("%s - Adding %s to plan", self.name,
                    "{}_stared".format(CallbackHopsworks.__name__))
        if StartServices.has_started_enabled_all_services(self.config):
            return []
        raise Exception(
            "{} - Services have not been enabled/started".format(self.name))

    def rollback(self):
        pass

    @staticmethod
    def has_started_enabled_all_services(config: configparser.ConfigParser) -> bool:
        kagent_conf_dir = config.get(
            'general', 'kagent_conf_dir', fallback="/srv/hops/kagent/etc")
        services_file = pathlib.Path(kagent_conf_dir, "services")
        services = configparser.ConfigParser()
        services.read(str(services_file))
        for section in services.sections():
            service = services.get(
                section, 'service', fallback='service_does_not_exist')
            if service != 'service_does_not_exist':
                if not StartServices.is_service_enabled(service) or not StartServices._is_service_running(service):
                    return False
        return True

    @staticmethod
    def _is_service_running(service: str) -> bool:
        output = SystemUtils.execute_command(
            ["systemctl", "is-active", "--quiet", service], check=False)
        return output.returncode == 0

    @staticmethod
    def is_service_enabled(service: str) -> bool:
        output = SystemUtils.execute_command(
            ["systemctl", "is-enabled", "--quiet", service], check=False)
        return output.returncode == 0

    @staticmethod
    def is_service_installed(service: str) -> bool:
        unit_file = pathlib.Path(f"/lib/systemd/system/{service}.service")
        return unit_file.is_file()

    def _start_service(self, service: str) -> None:
        SystemUtils.execute_command(
            ["systemctl", "start", service], timeout=120)

    def _stop_service(self, service: str) -> None:
        SystemUtils.execute_command(
            ["systemctl", "stop", service], timeout=120)

    def _reload_consul(self) -> None:
        SystemUtils.execute_command(["systemctl", "reload", "consul"])

    def _enable_service(self, service: str) -> None:
        SystemUtils.execute_command(
            ["systemctl", "enable", service], timeout=120)
            
    def _disable_service(self, service: str) -> None:
        SystemUtils.execute_command(
            ["systemctl", "disable", service], timeout=120)

==> ec2init-0.17-py3-none-any.whl: ec2init/plans/init/start_services.py <==
import logging
import configparser
import pathlib
from pathlib import Path

from typing import List, Tuple

from ec2init import Procedure
from ec2init.utils.system import SystemUtils
from ec2init.utils.dns import DnsUtils
from ec2init.plans.init.callback_hopsworks import CallbackHopsworks
from ec2init.utils.cloud import get_cloud_provider

logger = logging.getLogger(__name__)

class ServiceNotEnabledException(Exception):
    pass

class ServiceNotRunningException(Exception):
    pass

class StartServices(Procedure):
    SYSTEMCTL_ENABLE = "systemctl enable {service}"
    SYSTEMCTL_START = "systemctl start {service}"
    
    def __init__(self, name: str, gateway: str, instance_id: str, node_id: str, instance_type: str,
                 uid: str, config: configparser.ConfigParser, report_endpoint: Tuple[str, str], api_key: str, cloud_provider: str, search_domain: str,
                 check_file: Path = None) -> None:
        self.name = name
        self.gateway = gateway
        self.instance_id = instance_id
        self.uid = uid
        self.config = config
        self.report_endpoint = report_endpoint
        self.api_key = api_key
        self.node_id = node_id
        self.instance_type = instance_type
        self.cloud_provider = cloud_provider
        self.search_domain = search_domain
        self.check_file = check_file

    def pre(self) -> None:
        if self.report_callback is not None:
            try:
                self.report_callback.execute()
            except Exception as ex:
                logger.warning(
                    "%s - Could not execute pre-execute: %s", self.name, ex)

    def execute(self) -> List[Procedure]:
        logger.info("%s - Starting execution", self.name)

        if StartServices.has_started_enabled_all_services(self.config):
            # the service are already started, nothing to do
            if self.check_file is not None:
                self.check_file.touch()
            return []


        # Wait until internal hostname is resolved successfully
        # Get my FQDN
        fqdn = get_cloud_provider(self.cloud_provider).get_fqdn(self.config, self.search_domain, self.node_id)
        DnsUtils.get_ip_from_name(fqdn)

        kagent_conf_dir = self.config.get(
            'general', 'kagent_conf_dir', fallback="/srv/hops/kagent/etc")
        services_file = pathlib.Path(kagent_conf_dir, "services")
        services = configparser.ConfigParser()
        services.read(str(services_file))
        for section in services.sections():
            service = services.get(
                section, 'service', fallback='service_does_not_exist')
            if service != 'service_does_not_exist':
                logger.debug("%s - Starting service: %s", self.name, service)
                StartServices.start_service(service)
                logger.info("%s - Started service: %s", self.name, service)
                logger.debug("%s - Enabling service: %s", self.name, service)
                self._enable_service(service)
                logger.info("%s - Enabled service: %s", self.name, service)
                logger.debug("%s - Reloading Consul", self.name)
                self._reload_consul()
        logger.info("%s - Adding %s to plan", self.name,
                    "{}_stared".format(CallbackHopsworks.__name__))
        if StartServices.has_started_enabled_all_services(self.config):
            if self.check_file is not None:
                self.check_file.touch()
            return []
        raise Exception(
            "{} - Services have not been enabled/started".format(self.name))

    def rollback(self):
        pass

    @staticmethod
    def has_started_enabled_all_services(config: configparser.ConfigParser) -> bool:
        try:
            StartServices.all_services_started_enabled(config)
            return True
        except Exception as e:
            if isinstance(e, ServiceNotEnabledException) or isinstance(e, ServiceNotRunningException):
                return False
            raise e

    @staticmethod
    def all_services_started_enabled(config: configparser.ConfigParser) -> None:
        kagent_conf_dir = config.get(
            'general', 'kagent_conf_dir', fallback="/srv/hops/kagent/etc")
        services_file = pathlib.Path(kagent_conf_dir, "services")
        services = configparser.ConfigParser()
        services.read(str(services_file))
        for section in services.sections():
            service = services.get(
                section, 'service', fallback='service_does_not_exist')
            if service != 'service_does_not_exist':
                if not StartServices.is_service_enabled(service):
                    msg = f"Service {service} is not enabled"
                    logger.error(msg)
                    raise ServiceNotEnabledException(msg)
                if not StartServices._is_service_running(service):
                    msg = f"Service {service} is not running"
                    logger.error(msg)
                    raise ServiceNotRunningException(msg)
                logger.info(f"Service {service} is running")

    @staticmethod
    def _is_service_running(service: str) -> bool:
        output = SystemUtils.execute_command(
            ["systemctl", "is-active", "--quiet", service], check=False)
        return output.returncode == 0

    @staticmethod
    def is_service_enabled(service: str) -> bool:
        output = SystemUtils.execute_command(
            ["systemctl", "is-enabled", "--quiet", service], check=False)
        return output.returncode == 0

    @staticmethod
    def is_service_installed(service: str) -> bool:
        unit_file = pathlib.Path(f"/lib/systemd/system/{service}.service")
        return unit_file.is_file()

    @staticmethod
    def start_service(service: str) -> None:
        SystemUtils.execute_command(
            ["systemctl", "start", service], timeout=120)

    @staticmethod
    def stop_service(service: str) -> None:
        SystemUtils.execute_command(
            ["systemctl", "stop", service], timeout=120)

    def _reload_consul(self) -> None:
        SystemUtils.execute_command(["systemctl", "reload", "consul"])

    def _enable_service(self, service: str) -> None:
        SystemUtils.execute_command(
            ["systemctl", "enable", service], timeout=120)

    @staticmethod    
    def disable_service(service: str) -> None:
        SystemUtils.execute_command(
            ["systemctl", "disable", service], timeout=120)
